@@ -318,8 +318,12 @@ sub run {
             my $second_version = $entries[1]->Version;
             if ($first_version and $second_version) {
                 tag 'latest-debian-changelog-entry-without-new-version'
-                  unless versions_gt($first_version, $second_version)
-                  or $entries[0]->Changes =~ /backport/i;
+                  unless versions_gt(
+                    $first_version =~ s/^([^:]+)://r,
+                    $second_version =~ s/^([^:]+)://r
+                  )
+                  or $entries[0]->Changes =~ /backport/i
+                  or $entries[0]->Source ne $entries[1]->Source;
                 tag 'latest-debian-changelog-entry-changed-to-native'
                   if $native_pkg and $second_version =~ m/-/;
             }
@@ -328,10 +332,10 @@ sub run {
             $first_upstream =~ s/-[^-]+$//;
             my $second_upstream = $second_version;
             $second_upstream =~ s/-[^-]+$//;
-            my $first_debian = $first_version;
-            $first_debian =~ s/^[^-]+-//;
-            my $second_debian = $second_version;
-            $second_debian =~ s/^[^-]+-//;
+            my $first_debian =substr $first_version, length($first_upstream);
+            $first_debian =~ s/-([^-]+)$/$1/ if length($first_debian) > 0;
+            my $second_debian =substr $second_version,length($second_upstream);
+            $second_debian =~ s/-([^-]+)$/$1/ if length($second_debian) > 0;
 
             if ($first_upstream eq $second_upstream) {
                 if ($entries[0]->Changes
@@ -478,9 +482,12 @@ sub run {
         # length check.
         my @lines = split("\n", decode('utf-8', $changes));
         for my $i (0 .. $#lines) {
+            my $line = $i + $chloff;
+            tag 'debian-changelog-line-too-short', $1, "(line $line)"
+              if $lines[$i] =~ /^   [*]\s(.{1,5})$/;
             if (length($lines[$i]) > 81
                 and $lines[$i] !~ /^[\s.o*+-]*(?:[Ss]ee:?\s+)?\S+$/) {
-                tag 'debian-changelog-line-too-long', 'line ' . ($i + $chloff);
+                tag 'debian-changelog-line-too-long', "line $line";
             }
         }
 
