@@ -39,6 +39,9 @@ use constant BLOCKSIZE => 16_384;
 use constant INSANE_LINE_LENGTH => 512;
 use constant SAFE_LINE_LENGTH => 256;
 
+use File::Basename qw(basename);
+use List::MoreUtils qw(any);
+
 use Lintian::Data;
 use Lintian::Relation ();
 use Lintian::Tags qw(tag);
@@ -151,6 +154,9 @@ my $RFC_WHITELIST =  Lintian::Data->new(
         return qr/$_[0]/xms;
     });
 
+# "Known good" files that match eg. lena.jpg.
+my $LENNA_WHITELIST = Lintian::Data->new('cruft/lenna-whitelist');
+
 # prebuilt-file or forbidden copyright
 my $BAD_LINK_COPYRIGHT =  Lintian::Data->new(
     'cruft/bad-link-copyright',
@@ -333,7 +339,7 @@ our @TRAILING_WHITESPACE_FILES = (
 );
 
 sub run {
-    my (undef, undef, $info, $proc) = @_;
+    my (undef, undef, $info, $proc, $group) = @_;
     my $source_pkg = $proc->pkg_src;
     my $d_files = $info->index_resolved_path('debian/files');
 
@@ -341,12 +347,15 @@ sub run {
         tag 'debian-files-list-in-source';
     }
 
+    tag 'package-uses-deprecated-source-override-location'
+      if $info->index_resolved_path('debian/source.lintian-overrides');
+
     # This doesn't really belong here, but there isn't a better place at the
     # moment to put this check.
     my $version = $info->field('version', '0-1');
 
     if ($info->native) {
-        if ($version =~ /-/ and $version !~ /-0\.[^-]+$/) {
+        if ($version =~ /-/) {
             tag 'native-package-with-dash-version';
         }
     }else {
@@ -370,7 +379,7 @@ sub run {
     }elsif (not $info->native) {
         check_diffstat($info->diffstat, \%warned);
     }
-    find_cruft($source_pkg, $info, \%warned, $ltinbd);
+    find_cruft($source_pkg, $info, \%warned, $ltinbd, $group);
 
     for my $file (@EOL_TERMINATORS_FILES) {
         my $path = $info->index_resolved_path("debian/$file");
@@ -562,9 +571,10 @@ sub istestset {
 # "source-contains" tag.  The tag isn't entirely accurate, but it's better
 # than creating yet a third set of tags, and this gets the severity right.
 sub find_cruft {
-    my ($source_pkg, $info, $warned, $ltinbd) = @_;
+    my ($source_pkg, $info, $warned, $ltinbd, $group) = @_;
     my $prefix = ($info->native ? 'diff-contains' : 'source-contains');
     my @worklist;
+    my $ships_examples = _ships_examples($group);
 
     # start with the top-level dirs
     push(@worklist, $info->index('')->children);
@@ -602,6 +612,8 @@ sub find_cruft {
                         last;
                     }
                 }
+                tag 'package-does-not-install-examples', $entry
+                  if $basename eq 'examples' and not $ships_examples;
             }
 
             push(@worklist, $entry->children);
@@ -683,6 +695,7 @@ sub find_cruft {
             && $entry->is_open_ok) {
             my $fd = $entry->open;
             while (my $line = <$fd>) {
+                next if $line =~ m{^\s*dnl};
                 tag 'autotools-pkg-config-macro-not-cross-compilation-safe',
                   $name, "(line $.)"
                   if $line=~ m{AC_PATH_PROG\s*\([^,]+,\s*\[?pkg-config\]?\s*,};
@@ -697,7 +710,8 @@ sub find_cruft {
                 or $file_info =~ /\bbitmap\b/i
                 or $file_info =~ /^PDF Document\b/i
                 or $file_info =~ /^Postscript Document\b/i) {
-                tag 'license-problem-non-free-img-lenna', $name;
+                tag 'license-problem-non-free-img-lenna', $name
+                  unless $LENNA_WHITELIST->known($md5sum);
             }
 
         }
@@ -818,6 +832,7 @@ sub check_missing_source {
     my ($file, $info, $name, $basename, $dirname,$replacementspairref,
         $extratext)
       = @_;
+    my $basename_of_dirname = basename($dirname);
     $extratext //= '';
 
     # do not check missing source for non free
@@ -856,6 +871,7 @@ sub check_missing_source {
             my $newpath;
             # first replace dir name
             $path =~ s/\$dirname/$dirname/g;
+            $path =~ s/\$basename_of_dirname/$basename_of_dirname/g;
             # absolute path
             if(substr($path,0,1) eq '/') {
                 $path =~ s,^/+,,g;
@@ -878,8 +894,8 @@ sub check_missing_source {
             if($newpath eq '') {
                 next PATH;
             }
-            # found source return
-            if($info->index($newpath)) {
+            # found source file or directory
+            if($info->index_resolved_path($newpath)) {
                 return;
             }
         }
@@ -929,11 +945,11 @@ sub full_text_check {
         # permission to distribute those.
         if(
             _license_check(
-                $source_pkg, $name,
-                $basename,$NON_DISTRIBUTABLE_LICENSES,
-                $block,$blocknumber,
-                \$cleanedblock,\%matchedkeyword,
-                \%licenseproblemhash
+                $info, $source_pkg,
+                $name,$basename,
+                $NON_DISTRIBUTABLE_LICENSES,$block,
+                $blocknumber,\$cleanedblock,
+                \%matchedkeyword,\%licenseproblemhash
             )
           ){
             return;
@@ -946,9 +962,10 @@ sub full_text_check {
         }
 
         _license_check(
-            $source_pkg, $name, $basename,
-            $NON_FREE_LICENSES,$block,  $blocknumber,
-            \$cleanedblock, \%matchedkeyword,\%licenseproblemhash
+            $info, $source_pkg, $name,
+            $basename,$NON_FREE_LICENSES,$block,
+            $blocknumber,\$cleanedblock, \%matchedkeyword,
+            \%licenseproblemhash
         );
 
         # check only in block 0
@@ -1010,6 +1027,11 @@ sub _search_in_block0 {
         elsif ($block =~ m/css_browser_selector\(/) {
             return;
         }
+        # Avoid false-positives in Jush's syntax highlighting definition files.
+        elsif ($block =~ m/jush\.tr\./) {
+            return;
+        }
+
         # now search hidden minified
         _linelength_test($entry, $info, $name, $basename, $dirname,$block);
     }
@@ -1162,10 +1184,10 @@ sub _tag_gfdl {
 # return True in case of license problem
 sub _check_gfdl_license_problem {
     my (
-        $source_pkg, $name,$basename,
-        $block,$blocknumber,$cleanedblock,
-        $matchedkeyword,$licenseproblemhash,$licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
     my $rawgfdlsections  = $matchedhash{rawgfdlsections}  || '';
     my $rawcontextbefore = $matchedhash{rawcontextbefore} || '';
@@ -1275,10 +1297,10 @@ sub _check_gfdl_license_problem {
 # whitelist good rfc
 sub _rfc_whitelist_filename {
     my (
-        $source_pkg, $name, $basename,
-        $block,$blocknumber, $cleanedblock,
-        $matchedkeyword,$licenseproblemhash, $licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
     return 0 if $name eq 'debian/copyright';
     my $lcname = lc($basename);
@@ -1296,12 +1318,19 @@ sub _rfc_whitelist_filename {
 # whitelist php source
 sub _php_source_whitelist {
     my (
-        $source_pkg, $name, $basename,
-        $block,$blocknumber, $cleanedblock,
-        $matchedkeyword,$licenseproblemhash, $licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
 
+    my $copyright_path = $info->index_resolved_path('debian/copyright');
+    if (    $copyright_path
+        and $copyright_path->file_contents
+        =~ m{^Source: https?://pecl.php.net/package/.*$}m) {
+        return 0;
+    }
+
     if($source_pkg =~ m,^php\d*(?:\.\d+)?$,xms) {
         return 0;
     }
@@ -1405,6 +1434,8 @@ sub _clean_block {
 
     # Texinfo end tag (could be more clever but brute force is fast)
     $text =~ s/}/ /gxms;
+    # Tex section titles
+    $text =~ s/^\s*\\(sub)*section\*?\{\s*\S+/ /gxms;
     # single char at end
     # String, C-style comment/javadoc indent,
     # quotes for strings, pipe and backslash, tilde in some txt
@@ -1457,9 +1488,10 @@ sub _md5sum_based_check {
 # check bad license
 sub _license_check {
     my (
-        $source_pkg, $name, $basename,
-        $licensesdatas, $block, $blocknumber,
-        $cleanedblock,$matchedkeyword, $licenseproblemhash
+        $info, $source_pkg, $name,
+        $basename,$licensesdatas, $block,
+        $blocknumber,$cleanedblock,$matchedkeyword,
+        $licenseproblemhash
     )= @_;
     my $ret = 0;
 
@@ -1514,10 +1546,10 @@ sub _license_check {
 
         if(defined($licenseproblemdata->{'callsub'})) {
             my $subresult= $licenseproblemdata->{'callsub'}->(
-                $source_pkg, $name, $basename,
-                $block,$blocknumber,$cleanedblock,
-                $matchedkeyword,$licenseproblemhash,$licenseproblem,
-                %+
+                $info, $source_pkg, $name,
+                $basename,$block,$blocknumber,
+                $cleanedblock,$matchedkeyword,$licenseproblemhash,
+                $licenseproblem,%+
             );
             if($subresult) {
                 $licenseproblemhash->{$licenseproblem} = 1;
@@ -1534,6 +1566,19 @@ sub _license_check {
     return $ret;
 }
 
+sub _ships_examples {
+    my ($group) = @_;
+    my @procs = $group->get_processables('binary');
+    return 1 if not @procs;
+    foreach my $binpkg (@procs) {
+        my $name = $binpkg->pkg_name;
+        return 1 if $name =~ m{-examples$};
+        my @files = $binpkg->info->sorted_index;
+        return 1 if any { m{^usr/share/doc/$name/examples/$} } @files;
+    }
+    return;
+}
+
 1;
 
 # Local Variables:
