@@ -43,6 +43,7 @@ use constant {
 };
 
 our $KNOWN_ESSENTIAL = Lintian::Data->new('fields/essential');
+our $KNOWN_TOOLCHAIN = Lintian::Data->new('fields/toolchain');
 our $KNOWN_METAPACKAGES = Lintian::Data->new('fields/metapackages');
 our $NO_BUILD_DEPENDS = Lintian::Data->new('fields/no-build-depends');
 our $KNOWN_SECTIONS = Lintian::Data->new('fields/archive-sections');
@@ -103,6 +104,9 @@ my $NAME_SECTION_MAPPINGS = Lintian::Data->new(
         return {'regex' =>  qr/$_[0]/x, 'section' => $_[1]};
     });
 
+our $DH_ADDONS = Lintian::Data->new('common/dh_addons', '=');
+our %DH_ADDONS_VALUES = map { $DH_ADDONS->value($_) => 1 } $DH_ADDONS->all;
+
 my %VCS_EXTRACT = (
     browser => sub { return @_;},
     arch    => sub { return @_;},
@@ -125,6 +129,9 @@ my %VCS_CANONIFY = (
                   {https://anonscm.debian.org/git/};
         $_[0] =~ s{https?\Q://bzr.debian.org/loggerhead/\E}
                   {https://anonscm.debian.org/loggerhead/};
+        $_[0] =~ s{https?\Q://salsa.debian.org/\E([^/]+/[^/]+)\.git/?$}
+                  {https://salsa.debian.org/$1};
+
         if ($_[0] =~ m{https?\Q://anonscm.debian.org/viewvc/\E}xsm) {
             if ($_[0] =~ s{\?(.*[;\&])?op=log(?:[;\&](.*))?\Z}{}xsm) {
                 my (@keep) = ($1, $2, $3);
@@ -168,6 +175,8 @@ my %VCS_CANONIFY = (
                   {https://anonscm.debian.org/git/};
         $_[0] =~ s{\Qgit://anonscm.debian.org/git/\E}
                   {https://anonscm.debian.org/git/};
+        $_[0] =~ s{https?\Q://salsa.debian.org/\E([^/]+/[^/\.]+)(?!\.git)$}
+                  {https://salsa.debian.org/$1.git};
     },
     hg      => sub {
         $_[0] =~ s{https?\Q://hg.debian.org/\E}
@@ -250,6 +259,7 @@ sub run {
             unfold('package', \$name);
             tag 'bad-package-name' unless $name =~ /^$PKGNAME_REGEX$/i;
             tag 'package-not-lowercase' if ($name =~ /[A-Z]/);
+            tag 'unusual-documentation-package-name' if $name =~ /-docs$/;
         }
     }
 
@@ -354,10 +364,21 @@ sub run {
             $arch =~ s/\n//o;
             tag 'illegal-multi-arch-value', $arch, $march if ($arch eq 'all');
         }
+
+    }
+
+    if ($type eq 'source') {
+        for my $bin ($info->binaries) {
+            my $arch = $info->binary_field($bin, 'architecture');
+            my $fname = "debian/$bin.lintian-overrides.$arch";
+            next unless $info->binary_field($bin, 'multi-arch', '') eq 'same';
+            tag 'multi-arch-same-package-has-arch-specific-overrides', $fname
+              if $info->index_resolved_path($fname);
+        }
     }
 
     if ($type eq 'binary'){
-        if ($pkg =~ /^fonts-/) {
+        if ($pkg =~ /^x?fonts-/) {
             tag 'font-package-not-multi-arch-foreign'
               unless $info->field('multi-arch', 'no')
               =~ m/^(?:foreign|allowed)$/o;
@@ -418,12 +439,16 @@ sub run {
     #---- Maintainer
     #---- Uploaders
 
+    my $is_comaintained = 0;
     for my $f (qw(maintainer uploaders)) {
         if (not defined $info->field($f)) {
             tag 'no-maintainer-field' if $f eq 'maintainer';
         } else {
             my $maintainer = $info->field($f);
 
+            my $is_list = $maintainer =~ /\@lists(?:\.alioth)?\.debian\.org\b/;
+            $is_comaintained = 1 if $is_list;
+
             # Note, not expected to hit on uploaders anymore, as dpkg
             # now strips newlines for the .dsc, and the newlines don't
             # hurt in debian/control
@@ -439,6 +464,7 @@ sub run {
                 my @uploaders = map { split /\@\S+\K\s*,\s*/ }
                   split />\K\s*,\s*/, $maintainer;
                 for my $uploader (@uploaders) {
+                    $is_comaintained = 1;
                     check_maintainer($uploader, 'uploader');
                     if (   ((true { $_ eq $uploader } @uploaders) > 1)
                         and($duplicate_uploaders{$uploader}++ == 0)) {
@@ -448,7 +474,7 @@ sub run {
             } else {
                 check_maintainer($maintainer, $f);
                 if (   $type eq 'source'
-                    && $maintainer =~ /\@lists(?:\.alioth)?\.debian\.org\b/
+                    && $is_list
                     && !defined $info->field('uploaders')) {
                     tag 'no-human-maintainers';
                 }
@@ -877,6 +903,13 @@ sub run {
                         && $pkg !~ m/-(?:dev|docs?|tools|bin)$/
                         && $part_d_orig =~ m/-docs?$/);
 
+                    tag 'binary-package-depends-on-toolchain-package',
+                      "$field: $part_d_orig"
+                      if $KNOWN_TOOLCHAIN->known($d_pkg)
+                      and not $pkg =~ m/^dh-/
+                      and not $pkg =~ m/-(source|src)$/
+                      and not $DH_ADDONS_VALUES{$pkg};
+
                     # default-jdk-doc must depend on openjdk-X-doc (or
                     # classpath-doc) to be useful; other packages
                     # should depend on default-jdk-doc if they want
@@ -1236,8 +1269,9 @@ sub run {
         tag 'redundant-bugs-field'
           if $bugs =~ m,^debbugs://bugs.debian.org/?$,i;
 
-        tag 'bugs-field-does-not-refer-to-debian-infrastructure', "line $."
-          if $bugs !~ m,\.debian\.org,;
+        tag 'bugs-field-does-not-refer-to-debian-infrastructure', $bugs,
+          "(line $.)"
+          unless $bugs =~ m,\.debian\.org, or $pkg =~ /[-]dbgsym$/;
     }
 
     #----- Python-Version
@@ -1333,21 +1367,35 @@ sub run {
                     tag $tag, $parts[0], $canonicalized;
                 }
             }
-            tag 'vcs-browser-links-to-empty-view', $uri
-              if $vcs eq 'browser' and $uri =~ m%rev=0&sc=0%;
-            $seen_vcs{$vcs}++ if $vcs ne 'browser';
+            if ($vcs eq 'browser') {
+                tag 'vcs-browser-links-to-empty-view', $uri
+                  if $uri =~ m%rev=0&sc=0%;
+                next;
+            }
+            $seen_vcs{$vcs}++;
+            if ($uri =~ m/\.debian\.org/) {
+                tag 'vcs-deprecated-in-debian-infrastructure', "vcs-$vcs", $uri
+                  if $vcs ne 'git';
+            } else {
+                tag 'orphaned-package-not-maintained-in-debian-infrastructure',
+                  "vcs-$vcs", $uri
+                  if $info->field('maintainer', '')
+                  =~ /packages\@qa.debian.org/;
+            }
         }
     }
     tag 'vcs-fields-use-more-than-one-vcs', sort keys %seen_vcs
       if keys %seen_vcs > 1;
 
+    tag 'co-maintained-package-with-no-vcs-fields'
+      if $type eq 'source'
+      and $is_comaintained
+      and not %seen_vcs;
+
     #---- Checksums
 
-    if ($type eq 'source') {
-        if (!$info->field('checksums-sha256')) {
-            tag 'no-strong-digests-in-dsc';
-        }
-    }
+    tag 'no-strong-digests-in-dsc'
+      if $type eq 'source' && !$info->field('checksums-sha256');
 
     #----- Field checks (without checking the value)
 
