@@ -139,12 +139,19 @@ sub _load_file_package_list_mapping {
         sub {
             my $pkg = strip($_[0]);
             my $pkg_regexp = qr/^$pkg$/x;
-            my $file_regexp = strip($_[1]);
+            my @sliptline = split(/\s*\~\~/, $_[1], 2);
+            my $file_regexp = strip($sliptline[0]);
             $file_regexp =~ s/\$EXT/$ext/g;
+            my $recontents = $reinside;
+            if (scalar(@sliptline) == 2) {
+                my $contents = strip($sliptline[1]);
+                $recontents = qr/$contents/;
+            }
             return {
                 'pkg_re' => $pkg_regexp,
                 'pkg' => $pkg,
                 'match' => qr/$file_regexp/,
+                'contents_re' => $recontents,
             };
         });
     return {
@@ -152,7 +159,6 @@ sub _load_file_package_list_mapping {
         'mapping' => $mapping,
         'ext' => $ext,
         'tag' => $tagname,
-        'reinside' => $reinside,
     };
 }
 
@@ -210,13 +216,13 @@ sub _detect_embedded_libraries {
         if($fname =~ m/$typere/) {
             my $mapping = $type->{'mapping'};
             my $typetag = $type->{'tag'};
-            my $reinside = $type->{'reinside'};
           LIBRARY:
             foreach my $library ($mapping->all) {
                 my $library_data = $mapping->value($library);
                 my $mainre = $library_data->{'pkg_re'};
                 my $mainpkg = $library_data->{'pkg'};
                 my $filere = $library_data->{'match'};
+                my $reinside = $library_data->{'contents_re'};
                 unless ($fname =~ m,$filere,) {
                     next LIBRARY;
                 }
@@ -226,9 +232,7 @@ sub _detect_embedded_libraries {
                 if(defined($reinside)) {
                     my $foundre = 0;
                     my $fd = $file->open(':raw');
-
-                    my $sfd
-                      = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); });
+                    my $sfd = Lintian::SlidingWindow->new($fd);
 
                   READWINDOW:
                     while (my $block = $sfd->readwindow) {
@@ -1286,7 +1290,7 @@ sub run {
             and not $fname =~ m{ \. (?:
                   # Common "non-license" file extensions...
                    el|[ch]|cc|p[ylmc]|[hu]i|p_hi|html|php|rb|xpm
-                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml
+                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml|rs
                ) \Z}xsm
             and not $fname=~ m,^usr/share/zope/Products/.*\.(?:dtml|pt|cpt)$,
             and not $fname =~ m,/under\S+License\.docbook$,
@@ -2047,17 +2051,23 @@ sub _check_tag_url_privacy_breach {
                       >,xismog;
         my $relcontent = $1;
         if (defined($relcontent)) {
-            if ($relcontent eq 'schema.dct') {
-                # see #736992
-                return;
-            } elsif  ($relcontent eq 'bookmark') {
-                # see #746656
-                return;
-            } elsif ($relcontent eq 'generator-home') {
-                # generator-home is used by texinfo
-                return;
-                # reparse for alternate (css alternate is loaded)
-            } elsif ($relcontent eq 'alternate') {
+            # See, for example, https://www.w3schools.com/tags/att_link_rel.asp
+            my %allowed = (
+                'alternate'         => 1, # #891301
+                'author'            => 1, # #891301
+                'bookmark'          => 1, # #746656
+                'canonical'         => 1, # #762753
+                'generator'         => 1, # #891301
+                'generator-home'    => 1, # texinfo
+                'help'              => 1, # #891301
+                'license'           => 1, # #891301
+                'next'              => 1, # #891301
+                'prev'              => 1, # #891301
+                'schema.dct'        => 1, # #736992
+                'search'            => 1, # #891301
+            );
+            return if ($allowed{$relcontent});
+            if ($relcontent eq 'alternate') {
                 my $type = $fulltag;
                 $type =~ m,<link
                       (?:\s[^>]+)? \s+
@@ -2100,7 +2110,7 @@ sub _check_tag_url_privacy_breach {
 
     # generic case
     unless (exists $privacybreachhash->{'tag-generic-'.$website}){
-        tag 'privacy-breach-generic', $file, "($url)";
+        tag 'privacy-breach-generic', $file, "[$fulltag] ($url)";
         $privacybreachhash->{'tag-generic-'.$website} = 1;
     }
     return;
@@ -2184,11 +2194,12 @@ sub detect_privacy_breach {
             if(index($block,$keyword) > -1) {
                 my $keyvalue = $PRIVACY_BREAKER_FRAGMENTS->value($keyword);
                 my $regex = $keyvalue->{'regex'};
-                if ($block =~ m{$regex}) {
+                if ($block =~ m{($regex)}) {
+                    my $capture = $1;
                     my $breaker_tag = $keyvalue->{'tag'};
                     unless (exists $privacybreachhash{'tag-'.$breaker_tag}){
                         $privacybreachhash{'tag-'.$breaker_tag} = 1;
-                        tag $breaker_tag, $file;
+                        tag $breaker_tag, $file, "(choke on: $capture)";
                     }
                 }
             }
