@@ -166,7 +166,7 @@ sub run {
     my @indep_rules = (qr/^build$/, qr/^build-indep$/, qr/^binary-indep$/);
     my (@current_targets, %rules_per_target,  %debhelper_group);
     my (%seen, %overridden);
-    my $maybe_skipping;
+    my ($maybe_skipping, @conditionals);
     my %variables;
     my $uses_makefile_pl = 0;
     my $includes = 0;
@@ -177,6 +177,9 @@ sub run {
         }
         my $line = $_;
 
+        tag 'debian-rules-is-dh_make-template'
+          if $line =~ m/dh_make generated override targets/;
+
         next if /^\s*\#/;
         if (m/^\s*[s-]?include\s+(\S++)/o){
             my $makefile = $1;
@@ -230,7 +233,11 @@ sub run {
         foreach my $bad_construct ($BAD_CONSTRUCT_IN_RULES->all) {
             my $badregex = $BAD_CONSTRUCT_IN_RULES->value($bad_construct);
             if ($line =~ m/$badregex/) {
-                tag $bad_construct, "line $.";
+                if (defined($+{info})) {
+                    tag $bad_construct, $+{info}, "(line $.)";
+                } else {
+                    tag $bad_construct, "line $.";
+                }
             }
         }
 
@@ -244,10 +251,13 @@ sub run {
             # rather well.
             my ($var, $value) = ($1, $2);
             $variables{$var} = $value;
+            tag 'unnecessary-source-date-epoch-assignment', "(line $.)"
+              if $var eq 'SOURCE_DATE_EPOCH';
         }
 
         # Keep track of whether this portion of debian/rules may be optional
-        if (/^ifn?(?:eq|def)\s/) {
+        if (/^ifn?(?:eq|def)\s(.*)/) {
+            push(@conditionals, $1);
             $maybe_skipping++;
         } elsif (/^endif\s/) {
             $maybe_skipping--;
@@ -397,10 +407,9 @@ sub run {
                 } elsif ($typerule eq 'recommended_allindep') {
                     tag 'debian-rules-missing-recommended-target', $target;
                 } elsif ($typerule eq 'goodpractice_dfsg') {
-                    if ($version =~ /(dfsg|debian|ds)/) {
-                        tag 'debian-rules-missing-good-practice-target-dfsg',
-                          $target;
-                    }
+                    tag 'debian-rules-missing-good-practice-target-dfsg',
+                      $target
+                      if $info->repacked;
                 } else {
                     $typerule ||= '<N/A>';
                     croak(
@@ -426,12 +435,26 @@ sub run {
         tag 'binary-arch-rules-but-pkg-is-arch-indep' if $nonempty;
     }
 
-    foreach my $suffix (qw(dh_clean dh_fixperms)) {
-        my $line = $overridden{$suffix};
-        tag "override_$suffix-does-not-call-$suffix", "(line $line)"
-          if $line
-          and none { m/^\t\s*-?$suffix\b/ }
-        @{$rules_per_target{"override_$suffix"}};
+    foreach my $cmd (qw(dh_clean dh_fixperms)) {
+        foreach my $suffix ('', '-indep', '-arch') {
+            my $line = $overridden{"$cmd$suffix"};
+            tag "override_$cmd-does-not-call-$cmd", "(line $line)"
+              if $line
+              and none { m/^\t\s*-?($cmd\b|\$\(overridden_command\))/ }
+            @{$rules_per_target{"override_$cmd$suffix"}};
+        }
+    }
+
+    if (my $line = $overridden{'dh_auto_test'}) {
+        my @lines = grep {
+                  $_ !~ m{^\t\s*[\:\[]}
+              and $_ !~ m{\bdh_auto_test\b}
+              and $_
+              !~ m{^\t\s*[-@]?(?:(?:/usr)?/bin/)?(?:cp|chmod|echo|ln|mv|mkdir|rm|test|true)}
+        } @{$rules_per_target{'override_dh_auto_test'}};
+        tag 'override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
+          "(line $line)"
+          if @lines and none { m/(DEB_BUILD_OPTIONS|nocheck)/ } @conditionals;
     }
 
     # Make sure that all the required build dependencies are there.  Don't
