@@ -748,6 +748,7 @@ sub main {
         # Do not start a new group if we have a signal pending.
         retrigger_signal() if $received_signal;
 
+        v_msg("Starting on group $gname");
         my $total_raw_res = timed_task {
             my @group_lpkg;
             my $raw_res = timed_task {
@@ -808,6 +809,7 @@ sub main {
         } else {
             print {$STATUS_FD} "error ${gname}${total_tres}\n";
         }
+        v_msg("Finished processing group $gname");
     }
 
     # Write the lab state to the disk, so it remembers the new packages
@@ -1508,17 +1510,15 @@ sub setup_work_pool {
     for my $arg (@ARGV) {
         # file?
         if (-f $arg) {
-            if ($arg =~ m/\.(?:u?deb|dsc|changes)$/o){
+            if ($arg =~ m/\.(?:u?deb|dsc|changes|buildinfo)$/o){
                 eval {$pool->add_file($arg);};
                 if ($@) {
                     print STDERR "Skipping $arg: $@";
                     $exit_code = 2;
                 }
             } else {
-                fatal_error(
-                    join(q{ },
-                        "bad package file name $arg",
-                        '(neither .deb, .udeb, .changes or .dsc file)'));
+                fatal_error("bad package file name $arg (neither .deb, "
+                      . '.udeb, .changes .dsc or .buildinfo file)');
             }
         } else {
             # parameter is a package name--so look it up
