@@ -65,6 +65,7 @@ BEGIN {
           file_is_encoded_in_non_utf8
           is_string_utf8_encoded
           fail
+          internal_error
           do_fork
           run_cmd
           strip
@@ -92,6 +93,7 @@ BEGIN {
           pipe_tee
           untaint
           $PKGNAME_REGEX
+          $PKGREPACK_REGEX
           $PKGVERSION_REGEX
           ),
         @{ $EXPORT_TAGS{constants} });
@@ -215,6 +217,15 @@ is not anchored and does not enforce any "boundary" characters.
 
 our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/o;
 
+=item $PKGREPACK_REGEX
+
+Regular expression that matches "repacked" package names.  The expression is
+not anchored and does not enforce any "boundary" characters.
+
+=cut
+
+our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)/o;
+
 =item $PKGVERSION_REGEX
 
 Regular expression that matches valid package versions.  The
@@ -1183,7 +1194,7 @@ sub touch_file {
     return 1;
 }
 
-=item fail (MSG[, ...])
+=item internal_error (MSG[, ...])
 
 Use to signal an internal error. The argument(s) will used to print a
 diagnostic message to the user.
@@ -1192,9 +1203,19 @@ If multiple arguments are given, they will be merged into a single
 string (by join (' ', @_)).  If only one argument is given it will be
 stringified and used directly.
 
+=item fail (MSG[, ...])
+
+Deprecated alias of "internal_error".
+
 =cut
 
 sub fail {
+    warnings::warnif('deprecated',
+        '[deprecation] fail() has been replaced by internal_error()');
+    goto \&internal_error;
+}
+
+sub internal_error {
     my $str = 'internal error: ';
     if (@_) {
         $str .= join ' ', @_;
@@ -1205,7 +1226,6 @@ sub fail {
             $str .= 'No context.';
         }
     }
-    $! = 2; # set return code outside eval()
     croak $str;
 }
 
@@ -1229,7 +1249,7 @@ If the tool cannot be found, this sub will cause a trappable error.
     sub locate_helper_tool {
         my ($toolname) = @_;
         if ($toolname =~ m{(?:\A|/) \.\. (?:\Z|/)}xsm) {
-            fail("$toolname is not a valid tool name");
+            internal_error("$toolname is not a valid tool name");
         }
         return $_CACHE{$toolname} if exists $_CACHE{$toolname};
 
@@ -1246,7 +1266,7 @@ If the tool cannot be found, this sub will cause a trappable error.
             }
         }
         $toolpath_str //= '<N/A>';
-        fail(
+        internal_error(
             sprintf(
                 'Cannot locate %s (search dirs: %s)',
                 $toolname, $toolpath_str
@@ -1710,11 +1730,23 @@ gcc-5 + gcc-5-cross + gcc-6 + gcc-6-cross).
 
 sub find_backlog {
     my ($lintian_version, $state) = @_;
-    my (@backlog, %by_version);
+    my (@backlog, %by_version, @low_priority);
     for my $group_id (keys(%{$state->{'groups'}})) {
         my $last_version = '0';
         my $group_data = $state->{'groups'}{$group_id};
         my $is_out_of_date;
+        # Does this group repeatedly fail with the current version
+        # of lintian?
+        if (    exists($group_data->{'processing-errors'})
+            and $group_data->{'processing-errors'} > 2
+            and exists($group_data->{'last-error-by'})
+            and $group_data->{'last-error-by'} ne $lintian_version) {
+            # To avoid possible "starvation", we will give lower priority
+            # to packages that repeatedly fail.  They will be retried as
+            # the backlog is cleared.
+            push(@low_priority, $group_id);
+            next;
+        }
         if (exists($group_data->{'out-of-date'})) {
             $is_out_of_date = $group_data->{'out-of-date'};
         }
@@ -1728,6 +1760,7 @@ sub find_backlog {
     for my $v (sort(versions_comparator keys(%by_version))) {
         push(@backlog, shuffle(@{$by_version{$v}}));
     }
+    push(@backlog, shuffle(@low_priority)) if @low_priority;
     return @backlog;
 }
 
